package javax.swing.text;

import sun.awt.AppContext;
import java.util.Vector;

public class LayoutQueue
{
    private static final Object DEFAULT_QUEUE;
    private Vector<Runnable> tasks;
    private Thread worker;
    
    public LayoutQueue() {
        this.tasks = new Vector();
    }
    
    public static LayoutQueue getDefaultQueue() {
        AppContext appContext = AppContext.getAppContext();
        synchronized (LayoutQueue.DEFAULT_QUEUE) {
            LayoutQueue value = (LayoutQueue)appContext.get(LayoutQueue.DEFAULT_QUEUE);
            if (value == null) {
                value = new LayoutQueue();
                appContext.put(LayoutQueue.DEFAULT_QUEUE, value);
            }
            return value;
        }
    }
    
    public static void setDefaultQueue(LayoutQueue value) {
        synchronized (LayoutQueue.DEFAULT_QUEUE) {
            AppContext.getAppContext().put(LayoutQueue.DEFAULT_QUEUE, value);
        }
    }
    
    public synchronized void addTask(Runnable obj) {
        if (this.worker == null) {
            (this.worker = new LayoutQueue.LayoutThread()).start();
        }
        this.tasks.addElement(obj);
        this.notifyAll();
    }
    
    protected synchronized Runnable waitForWork() {
        while (this.tasks.size() == 0) {
            try {
                this.wait();
            }
            catch (InterruptedException ex) {
                return null;
            }
        }
        Runnable runnable = (Runnable)this.tasks.firstElement();
        this.tasks.removeElementAt(0);
        return runnable;
    }
    
    static {
        DEFAULT_QUEUE = new Object();
    }
    
    class LayoutThread extends Thread
    {
        LayoutThread() {
            super("text-layout");
            this.setPriority(1);
        }
        
        @Override
        public void run() {
            Runnable waitForWork;
            do {
                waitForWork = LayoutQueue.this.waitForWork();
                if (waitForWork != null) {
                    waitForWork.run();
                }
                else {
                    continue;
                }
            } while (waitForWork != null);
        }
    }
}
